/***************************************************************************
                          dc.cpp  -  description
                             -------------------
    begin                : Wed Jan 14 11:54:40 EST 2004
    copyright            : (C) 2004 by Andrew Bibb
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dc.h"
#include "resource.h"

#include <qhgroupbox.h>
#include <qvgroupbox.h>
#include <qvbuttongroup.h>
#include <qstring.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qapplication.h>
#include <qwhatsthis.h>
#include <qfileinfo.h>
#include <qaccel.h>
#include <qdir.h>


#ifndef _TOOLKIT_IN_DLL_
#include "RxDynamicModule.h"

ODRX_DECLARE_STATIC_MODULE_ENTRY_POINT(ModelerModule);

ODRX_BEGIN_STATIC_MODULE_MAP()
  ODRX_DEFINE_STATIC_APPLICATION("ModelerGeometry",  ModelerModule)
ODRX_END_STATIC_MODULE_MAP()

#endif


dc_Dialog::dc_Dialog(QWidget* parent, const char* name, bool modal):QDialog(parent,name,modal)
{
	const short margin = 8;
	const int defaultver = 7;	//	rel 14 as default version to save
	const char* versions[] = {
												    "Version 2.5",
														"Version 2.6",
														"Version 9",
														"Version 10",
														"Version 11/12",
														"Version 13",
														"Version 14",
														"Version 2000",
														"Version 2004",
														0};
														
//	First out let's initialize system services
	ODRX_INIT_STATIC_MODULE_MAP();
	odInitialize (&dc_svcs);
	
//	Now let's create widgets
	p2fod = new QFileDialog(this, "file open dialog");
		p2fod->addFilter(tr("Drawing Interchange Files (*.dxf *.DXF)"));
		p2fod->addFilter(tr("Standard Drawing Files (*.dwg *.DWG)"));
		p2fod->setDir(QDir::homeDirPath());
		p2fod->setMode(QFileDialog::ExistingFiles);
		p2fod->setCaption(tr("Open File"));
		
	p2fsd = new QFileDialog(this, "file save dialog");
		p2fsd->setDir(QDir::homeDirPath());
		p2fsd->setCaption(tr("Save File"));
		p2fsd->setMode(QFileDialog::AnyFile);
		
	QHGroupBox* in_box = new QHGroupBox(tr("Input"),this, "input layout box");
		in_line = new QLineEdit(in_box, "input line edit");
		in_line->setReadOnly(true);
		in_browse = new QPushButton(tr("Browse"),in_box,"input browse button");
		
	QHGroupBox* out_box = new QHGroupBox(tr("Output"),this,"output layout box");
		out_line = new QLineEdit(out_box, "output line edit");
		out_line->setReadOnly(true);
		out_browse = new QPushButton(tr("Browse"),out_box,"output browse button");
	
	QVButtonGroup* format_box = new QVButtonGroup(tr("Output Format"),this,"format box");
		version_sel = new QComboBox(false, format_box, "version selector combo box");
		version_sel->insertStrList(versions);
		version_sel->setCurrentItem(defaultver);
		dwg_radbut = new QRadioButton(tr("DWG"), format_box, "dwg radio button");
		dxf_radbut = new QRadioButton(tr("DXF"), format_box, "dxf radio button");
		dwg_radbut->setChecked(true); //	set drawing by default
		QHBox* hb1 = new QHBox(format_box, "dxf spinbox and label");
		spin_label = new QLabel(hb1, "spin box label");
		spin_label->setText(tr("Precision "));
		spin_label->setAlignment(AlignRight | AlignVCenter);
		dxf_prec = new QSpinBox(6, 16, 1, hb1, "precision spinbox");
		dxf_prec->setWrapping(true);
		dxf_prec->setValue(8);
		toggleDXFOptions(1);	//	set drawing by default
		
	QVButtonGroup* misc_box = new QVButtonGroup(tr("Misc. Options"),this,"misc box");
		seperate1_ckbx = new QCheckBox(tr("Seperate Layouts to Individual Drawings"), misc_box,"seperate1");
		seperate2_ckbx = new QCheckBox(tr("Seperate Layouts and place all in Model Space"), misc_box,"seperate2");			
		seperate1_ckbx->setEnabled(false);
		seperate2_ckbx->setEnabled(false);

	credits = new QLabel(this, "credits");
		credits->setText(tr("Drawing Convert %1\ncopyright (c)2004\nAndrew J. Bibb").arg(VERSION));
		credits->setAlignment(AlignCenter);	
	odlic = new QLabel(this, "odcopyright");
		odlic->setText(tr("\n%1 library version %2 licensed to:\n%3").arg(dc_svcs.product(),dc_svcs.versionString(),ODLICENSE));
		odlic->setAlignment(AlignCenter);
			
	whatsthis_but = QWhatsThis::whatsThisButton(this);
	conv_but = new QPushButton(tr("Convert"),this, "convert button");
	cancel_but = new QPushButton(tr("Cancel"),this, "cancel button");
		
	QVBoxLayout* vbox = new QVBoxLayout(this, margin, -1, "main layout");
		vbox->addWidget(in_box);
		vbox->addWidget(out_box);
		QHBoxLayout* hbox1 = new QHBoxLayout(vbox,margin);
			hbox1->addWidget(format_box);
			hbox1->addStretch();
			hbox1->addWidget(misc_box);
		vbox->addStretch(0);
		vbox->addSpacing(margin);
		vbox->addWidget(credits);
		vbox->addWidget(odlic);
		QHBoxLayout* hbox2 = new QHBoxLayout(vbox,margin);
			hbox2->addWidget(whatsthis_but);
			hbox2->addStretch(0);
			hbox2->addWidget(conv_but);
			hbox2->addWidget(cancel_but);
	setCaption(tr("DConvert Options"));

//	create the console window register a function key to it and then and hide it
  conwin = new QTextEdit(0, "Console window");
		conwin->setReadOnly(true);
		conwin->setCaption(tr("Program Messages"));
		QDesktopWidget *d = QApplication::desktop();
    conwin->setGeometry(d->width()/6, d->height()/4 , d->width()*2/3, d->height()/2);
		conwin->hide();
	QAccel* f2acc1 = new QAccel(this);	//	F2 inside this	
		f2acc1->connectItem(f2acc1->insertItem(Key_F2),this, SLOT(toggleConsoleWindow()));
	QAccel* f2acc2 = new QAccel(conwin);	//	F2 inside of conwin	
		f2acc2->connectItem(f2acc2->insertItem(Key_F2),this, SLOT(toggleConsoleWindow()));
	
//	setup the "Whats This" help
	QWhatsThis::add(in_line,tr("<p>The name of the file to be read will be displayed here.  Select"
															" the <b>browse button</b> at right to pick files or enter a name"));
	QWhatsThis::add(in_browse,tr("<p>Use this button to open a file dialog box that can be used"
															" to select a single file or multiple files graphically"));
	QWhatsThis::add(out_line,tr("<p>The name of the file to be saved will be displayed here.  Select"
															" the <b>browse button</b> at right to pick a file or enter a name."));
	QWhatsThis::add(out_browse,tr("<p>Use this button to open a file dialog box that can be used"
															" to select a file or directory graphically. If multiple input files are selected "
															"then the dialog will only allow you to specify an output directory.  In this case "
															"the output names will be the same as the input names.<p>Copies of your original "
															"files will be saved as backups if the the output file already exists."));
	QWhatsThis::add(version_sel,tr("<p>Use this control to select the file version to be saved"));
	QWhatsThis::add(dwg_radbut,tr("<p>Place a check mark in this box to save as a standard <b>"
																"DWG</b> file"));
	QWhatsThis::add(dxf_radbut,tr("<p>Place a check mark in this box to save as a drawing"
																" interchange format <b>DXF</b> file.<p>Selecting this box"
																" will activate the precision spinbox below"));
	QWhatsThis::add(seperate1_ckbx,tr("<p>Place each layout from the source drawing into seperate output drawings"
																		" while maintaining Paper Space and Model Space entities"));
	QWhatsThis::add(seperate2_ckbx,tr("<p>Place each layout from the source drawing into seperate output drawings"
																		" and move all entities to Model Space"));
	QWhatsThis::add(dxf_prec,tr("<p>Use this control to specify the number of decimal places"
															" to carry when writing the DXF file"));
	QWhatsThis::add(credits,tr("<p>Structural Engineer extraordinaire and friend to architects everywhere"));
	QWhatsThis::add(odlic,tr("<p>OD library license holder.  If you are not this person or do not"
													" work for this person you may be using an illegal copy of this program"));
	QWhatsThis::add(whatsthis_but,tr("<p>Use this control to enter \"Whats this\" mode"));
	QWhatsThis::add(conv_but,tr("<p>Use this button to start the conversion"));
	QWhatsThis::add(cancel_but,tr("<p>Use this button to exit the program"));
	QWhatsThis::add(conwin,tr("<p>Window to display progress messages and status reports.<p>"
												"This window can be toggled up and down with the F2 key"));
											
												
//	now connect signals to slots
	connect (in_browse,SIGNAL(clicked()),this,SLOT(callOpenDialog()));
	connect (out_browse,SIGNAL(clicked()),this,SLOT(callSaveDialog()));
	connect (conv_but,SIGNAL(clicked()),this,SLOT(convertPressed()));
	connect (cancel_but,SIGNAL(clicked()),qApp,SLOT(quit()));
	connect (qApp,SIGNAL(lastWindowClosed()),qApp,SLOT(quit()));
	connect (dxf_radbut,SIGNAL(stateChanged(int)),this,SLOT(toggleDXFOptions(int)));
}

//	Destructor
dc_Dialog::~dc_Dialog()
{
	odUninitialize();
	delete conwin;
}

////////////////////////////////////////////////////////////////////
//		SLOTS
//////////////////////////////////////////////////////////////////////

void dc_Dialog::callOpenDialog()
{
	if (p2fod->exec() == QDialog::Accepted)
	{
		QStringList list = p2fod->selectedFiles();
		QStringList::Iterator it = list.begin();
		int filecount = 0;
		//	find out how many files we selected
		while(it != list.end()) {
			in_line->setText(*it);
			filecount++;
			++it;}
		if (filecount == 1){
			p2fsd->setMode(QFileDialog::AnyFile);
			p2fsd->setSelection(in_line->text());
			out_line->setText(p2fsd->selectedFile());}
		else {
			in_line->setText(tr("Multiple Files Selected"));
			p2fsd->setMode(QFileDialog::DirectoryOnly);
			p2fsd->setSelection(p2fsd->dirPath());
			out_line->setText(p2fsd->dirPath());}
	}
}

void dc_Dialog::callSaveDialog()
{
	if (dxf_radbut->isChecked()) p2fsd->addFilter(tr("Drawing Interchange Files (*.dxf *.DXF)"));
	if (dwg_radbut->isChecked()) p2fsd->addFilter(tr("Standard Drawing Files (*.dwg *.DWG)"));
	if (p2fsd->exec() == QDialog::Accepted)
	{
		out_line->setText(p2fsd->selectedFile());
		if (p2fsd->mode() == QFileDialog::DirectoryOnly)
			p2fsd->setDir(p2fsd->selectedFile());
	}
}

void dc_Dialog::toggleDXFOptions(int state)
{
	bool oo = false;	//	default state off
	if (state == 2) oo = true;
	spin_label->setEnabled(oo);
	dxf_prec->setEnabled(oo);
}

void dc_Dialog::toggleConsoleWindow()	{conwin->isVisible()? conwin->hide() : conwin->show();}


void dc_Dialog::convertPressed()
{
	conwin->clear();
	if (!conwin->isVisible()) toggleConsoleWindow();
	conwin->raise();
	qApp->processEvents();

//  Determine the file save type and file save version.
	OdDb::SaveType save_type = OdDb::kDwg;
	OdDb::DwgVersion save_ver  = OdDb::kDHL_CURRENT;
	
	if (dwg_radbut->isChecked()) save_type = OdDb::kDwg;
	if (dxf_radbut->isChecked()) save_type = OdDb::kDxf;
	
	switch(version_sel->currentItem())
	{
		case 0:
			save_ver = OdDb::vAC025; break;
		case 1:
			save_ver = OdDb::vAC026; break;
		case 2:
			save_ver = OdDb::vAC09;  break;
		case 3: 
			save_ver = OdDb::vAC10; break;
		case 4:
			save_ver = OdDb::vAC12; break;
		case 5:
			save_ver = OdDb::vAC13; break;
		case 6:
			save_ver = OdDb::vAC14; break;
		case 7:
			save_ver = OdDb::vAC15; break;
		case 8:
			save_ver = OdDb::vAC18; break;
	}
	
//	now convert each file in the list
	QStringList list = p2fod->selectedFiles();
	QStringList::Iterator it = list.begin();
	int good = 0;
	int bad = 0;
	while(it != list.end()) {
		in_line->setText(*it);
		convertFile(save_type,save_ver) ? good++ : bad++;
		++it;}
	conwin->append(tr("\n<b>Process Summary</b>"));
	conwin->append(tr("Files converted = %1").arg(good));
	conwin->append(tr("Failed conversions = %1").arg(bad));
	conwin->append(tr("Total files processed = %1").arg(good + bad));
	conwin->append(tr("\n(press F2 to close this window)\n\n"));
}
	

///////////////////////////////////////////////////////////////////
//	Functions

// function that will actually do the conversions.  Returns true if successful
// false if not. 
bool dc_Dialog::convertFile(OdDb::SaveType save_type, OdDb::DwgVersion save_ver)
{
//	check to see if we have a file to read
	if (in_line->text().isNull() || in_line->text().isEmpty()){
		conwin->append(tr("Error - no input file was provded\n"));
		qApp->processEvents();
		return false;}

//	we have text in the input field, make sure we can get and process the file
	QFileInfo* fi_in = new QFileInfo(in_line->text());
	if (fi_in->exists()) {conwin->append(tr("Checking that input file <b>%1</b> exists....yes").arg(fi_in->absFilePath()));
		qApp->processEvents();}
	else {
		conwin->append(tr("Error - <b>%1</b> does not exist\n").arg(fi_in->absFilePath()));
		qApp->processEvents();
		delete fi_in;
		return false;}

	if (fi_in->isReadable()) conwin->append(tr("Checking for read permission....granted"));
	else {
		conwin->append(tr("Error - Permission to read <b>%1</b> is not granted\n").arg(fi_in->absFilePath()));
		qApp->processEvents();
		delete fi_in;
		return false;}	

//  if we have multile files set the output file name to match the input name, the
//  paths could be different
	if (p2fsd->mode() == QFileDialog::DirectoryOnly)
		out_line->setText(p2fsd->dirPath() + "/" + fi_in->fileName());
		
//  Lets try to read the file
	conwin->append(tr("Attempting to read <b>%1</b>").arg(fi_in->absFilePath()));
	qApp->processEvents();
	try	{
		const bool allow_cp_conv = true; // placeholder - not yet implemented by DD
		const bool partial_load = false; // all or nothing - don't even try to deal with partial file loads 
		pDb = dc_svcs.readFile(fi_in->absFilePath(),allow_cp_conv,partial_load, Oda::kShareDenyNo);
		conwin->append(tr("File sucessfully read")); 
		qApp->processEvents();
		delete fi_in;}
	catch (OdError& e){
		conwin->append(tr("DWGdirect Error <b>%1</b> occured while reading the file\n").arg(dc_svcs.getErrorDescription(e.code()).c_str()));	
		qApp->processEvents();
		delete fi_in;
		return false;}
	catch (...)	{
		conwin->append(tr("Unknown Error occured while trying to read %1\n").arg(fi_in->absFilePath()));
		qApp->processEvents();
		delete fi_in;
		return false;}
		
//	now do output, try to make a backup file if output file already exists
	QFileInfo* fi_out = new QFileInfo(validateSelection(out_line->text()));
	QDir fdir = QDir(fi_out->dirPath(true));
	if (fi_out->exists())
	{
		conwin->append(tr("Checking to see if output file <b>%1</b> exists....yes").arg(fi_out->absFilePath()));
		qApp->processEvents();
		if (fi_out->isWritable()) {conwin->append(tr("Checking for write permission....granted"));
				qApp->processEvents();}
		else {
			conwin->append(tr("Error - Permission to write <b>%1</b> is not granted\n").arg(fi_out->absFilePath()));
			qApp->processEvents();
			delete fi_in;
			delete fi_out;
			return false;}

		QFileInfo* fi_bak = new QFileInfo((fi_out->absFilePath()).append('~'));
		if (fi_bak->exists()) // there is already a backup so try to delete it
		{
			conwin->append(tr("Checking to see if backup file <b>%1</b> exists....yes").arg(fi_bak->absFilePath()));
			qApp->processEvents();
			if (fdir.remove(fi_bak->fileName())){
				conwin->append(tr("Attempting to remove existing backup....success"));
				qApp->processEvents();}
			else {
				conwin->append(tr("Error - cannot delete existing backup file <b>%1</b>\n").arg(fi_bak->absFilePath()));
				qApp->processEvents();
				delete fi_in;
				delete fi_out;
				delete fi_bak;
				return false;}
			}
		else {
				conwin->append(tr("Checking to see if backup file <b>%1</b> exists....no").arg(fi_bak->absFilePath()));
				qApp->processEvents();}
//  now create the backup file which also deletes the existing output file
		if (fdir.rename(validateSelection(out_line->text()),fi_bak->fileName(),true)){
			conwin->append(tr("Creating backup file...success"));
			qApp->processEvents();}
		else {
			conwin->append(tr("Error - cannot create backup file\n"));
			qApp->processEvents();
			delete fi_in;
			delete fi_out;
			delete fi_bak;
			return false;}
		delete fi_bak;
	}  // close the if output file exists
	else
		conwin->append(tr("Checking to see if output file <b>%1</b> exists....no").arg(fi_out->absFilePath()));

// now lets try to save the file
	conwin->append(tr("Starting the file conversion...."));
	qApp->processEvents();
	try {
		OdWrFileBuf fb(fi_out->absFilePath());
		pDb->writeFile(&fb, save_type, save_ver, false, dxf_prec->value()); // false is no bitmaps saved
		conwin->append(tr("File <b>%1</b> successfully saved").arg(fi_out->absFilePath()));
		qApp->processEvents();
		delete fi_out;}
	catch (OdError& e){
		conwin->append(tr("DWGdirect Error <b>%1</b> occured while writing the file\n").arg(dc_svcs.getErrorDescription(e.code()).c_str()));	
		qApp->processEvents();
		delete fi_out;
		return false;}
	catch (...)	{
		conwin->append(tr("An Unknown Error occured while trying to write <b>%1</b>\n").arg(fi_out->absFilePath()));
		qApp->processEvents();
		delete fi_out;
		return false;}
			
	conwin->append(tr("Conversion Complete\n\n"));
	return true;
}

// function that will return the file selected and also does some validation checking on the name.
// The validation is really for text that is typed in the dialog box (files selected by picking
// should not need validation). If the name typed in the box has a file type, and it agrees with
// the type selected in the filter window then return the name as typed. Otherwise append a file
// extension to what was typed. The following are examples:
// typed: drawing returned: drawing.dwg -- if the filter is set to dwg
// typed: drawing.dwg returned: drawing.dwg -- if the filter is set to dwg
// typed: drawing.dwg returned: drawing.dwg.bmp -- if the filter is set to bmp
QString dc_Dialog::validateSelection(const QString& selectedfile)
{
	QFileInfo* fi = new QFileInfo(selectedfile);
	QString ext = fi->extension(false); // false because we only want the last extension
	ext = ext.upper();
	delete fi;

	if ((dxf_radbut->isChecked()) && (ext != "DXF")) return selectedfile + ".dxf";
		else if ((dwg_radbut->isChecked()) && (ext != "DWG")) return selectedfile + ".dwg";
			else return selectedfile;
}
